#include <stdio.h>
#include <stdlib.h>
#include "pico/stdlib.h"
#include "hardware/gpio.h"

volatile bool timer_fired = false;

int64_t alarm_callback(alarm_id_t id, void *user_data) {
    timer_fired = true;
    return 0; // could return a value in us to fire in the future
}

volatile uint64_t prev = 0;

#define BUFSIZE (1024)
uint16_t buf[BUFSIZE];
volatile uint32_t pos = 0;

void gpio_callback(uint gpio, uint32_t events) {
    uint64_t now = time_us_64();
    uint16_t d = now - prev;
    bool high = events & GPIO_IRQ_EDGE_FALL;
    prev = now;

    if(high && !(pos & 1)) pos++; // put HIGH timings at odd positions
    if(pos < BUFSIZE) // avoid overflow
	    buf[pos++] = d;
}

uint16_t times[2][6] = {
    {0, 250, 1250, 2500, 5000, 0xFFFF},
    {0, 250, 500, 0xFFFF}
};
char letters[2][6] = { "#._L\n", "$-\n"};

void printEdge(uint16_t d, bool high) {
    // Pick times and letters based on high parameter
    uint16_t *t = times[high?1:0];
    char *l = letters[high?1:0];
    
    // Find nearest time in t to d until we reach 0xFFFF
    int bestC = l[0], bestD = abs(d-t[0]);
    for(int i = 1; t[i] != 0xFFFF; i++) {
        if(abs(d-t[i]) < bestD) {
            bestC = l[i];
            bestD = abs(d-t[i]);
        }
    }
    putchar(bestC);
}

int main() {
    stdio_init_all();
    while(!stdio_usb_connected());

    while(true) {
        printf("Press any key to start.\n");
        getchar_timeout_us(100*1000000);
        printf("Started!\n");

        for(int i = 0; i < BUFSIZE; i++) buf[i] = 0; // Clear
        pos = 0;
        timer_fired = false;

        add_alarm_in_ms(5000, alarm_callback, NULL, false);

        gpio_set_irq_enabled_with_callback(0,
                                           GPIO_IRQ_EDGE_RISE | GPIO_IRQ_EDGE_FALL,
                                           true, &gpio_callback);

        while(!timer_fired && pos <= BUFSIZE) tight_loop_contents();
        
        gpio_set_irq_enabled(0, GPIO_IRQ_EDGE_RISE | GPIO_IRQ_EDGE_FALL, false);
        printf("Ended.\n");

        for(int i = 0; i < pos; i++) printEdge(buf[i], i&1);
    }
}
